#!/bin/sh
#
# Install bodhi's uBoot and debian rootfs on Kirkwood devices

# Copyright (c) 2016 Alberto Bursi <starshipeleven@outlook.com>
# Copyright (c) 2012 Jeff Doozan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# Version 2.0    [2016] Heavily revised and extended by Alberto Bursi to cover more devices
#                       and to run with its own toolbox
# Version 1.1    [1/24/2013] Revised by Daniel Richard G.
# Version 1.0    [7/14/2012] Initial Release

##### MAINTAINER TIPS:
#
# -if you want to add support for a device, you need to add a new variable UBOOT_NEWNAME_URL with its link
#  in the "Definitions" section just below this paragraph, for its uboot download link.
#  The uboot md5 file link is generated dynamically inside the "install_uboot" function
#  by adding to UBOOT_MTD0_BASE_MD5 you find below the device-specific name and other things.
#  To have the new device show up in device selection phase and to add some more device-specific configuration,
#  scroll down into the "MAIN" section and add a new entry in the device selection case -- esac,
#  look at existing entries and copycat them.
#
# -if you want to change rootfs link, you need to change the link below in ROOTFS_URL, and the md5 link in ROOTFS_MD5
#
# -this script splits up code in functions for modularity reasons, so that you do not need to copy-paste the same code
#  over and over and over. There are 5 classes of functions used here:
#           --generic utility functions-- that deal with some specific action that is repeated
#            many times throughout the script
#           --functions that run a step of the installation routine--, calling also generic utility functions.
#           --device or vendor-specific functions-- that deal with device or vendor-specific actions
#            (usually killing stock firmware daemons that get in the way)
#           --install macrofunctions-- that mostly call a list of functions to run through all the install steps.
#           --disabled functions--, these are a safe way to keep dead code that might be useful in the future
#            inside this script, they contain remains from Jeff original script,
#            or things that do not work for other reasons.
#
#
# -any edit to this script should be done with a unix-aware text editor that has sintax highlighting for the very least
#  I personally recommend KWrite because it also has autocomplete and can shrink up functions.
#  If you are on windows and you must edit it please use notepad++ that is unix-aware and will not screw up the text.
###########


##############################
####
#### Definitions used throughout the script
####
##############################

#rootfs variables (debian-kirkwood-rootfs-bodhi-latest.tar.bz2 )
ROOTFS_URL="https://bitly.com/2agPH5E"
ROOTFS_MD5="https://raw.githubusercontent.com/mibodhi/kirkwood/kirkwood-2016-r1/checksum/debian-kirkwood-rootfs-bodhi-latest.tar.bz2.md5"

ROOTFS_TAR="debian-kirkwood-rootfs-bodhi-latest.tar"
ROOTFS="/tmp/rootfs/$ROOTFS_TAR.bz2"


#uboot-related variables

# uboot.bodhi.latest.environment
UBOOT_ENVIRONMENT_TXT_URL="https://bitly.com/2akfqZj"
#
# uboot.bodhi.latest.environment.img
UBOOT_ENVIRONMENT_IMG_URL="https://bitly.com/2akeSCD"
#
# uboot.bodhi.latest.environment.64K.img
UBOOT_ENVIRONMENT_IMG_64K_URL="https://bitly.com/2abh2oe"
#
# uboot.bodhi.latest.dockstar.mtd0.kwb
UBOOT_DOCKSTAR_URL="https://bitly.com/2aDBfsm"
#
# uboot.bodhi.latest.goflexhome.mtd0.kwb
UBOOT_GOFLEXHOME_URL="https://bitly.com/2aDBPGs"
#
# uboot.bodhi.latest.goflexnet.mtd0.kwb
UBOOT_GOFLEXNET_URL="https://bitly.com/2ayyzuS"
#
# uboot.bodhi.latest.iconnect.mtd0.kwb
UBOOT_ICONNECT_URL="https://bitly.com/2a6M5WP"
#
# uboot.bodhi.latest.netgear_ms2110.mtd0.kwb
UBOOT_NETGEAR_MS2110_URL="https://bitly.com/2al8b6E"
#
# uboot.bodhi.latest.nsa310.mtd0.kwb
UBOOT_NSA310_URL="https://bitly.com/2agOXxN"
#
# uboot.bodhi.latest.nsa310s.mtd0.kwb
UBOOT_NSA310S_URL="https://bitly.com/2a6LPax"
#
# uboot.bodhi.latest.nsa320.mtd0.kwb
UBOOT_NSA320_URL="https://bitly.com/2aDBT9w"
#
# uboot.bodhi.latest.nsa325.mtd0.kwb
UBOOT_NSA325_URL="https://bitly.com/2ayyyY5"
#
# uboot.bodhi.latest.pogo_e02.mtd0.kwb
UBOOT_POGO_E02_URL="https://bitly.com/2ayyCah"
#
# uboot.bodhi.latest.pogo_v4.mtd0.kwb
UBOOT_POGO_V4_URL="https://bitly.com/2aRl1YM"
#
# uboot.bodhi.latest.sheevaplug.mtd0.kwb
UBOOT_SHEEVAPLUG_URL="https://bitly.com/2aBZuFC"
#
# uboot.bodhi.latest.t5325.mtd0.kwb
UBOOT_T5325_URL="https://bitly.com/2aFz3h0"

#https://raw.githubusercontent.com/mibodhi/kirkwood/kirkwood-2016-r1/checksum/uboot.bodhi.latest.dockstar.mtd0.kwb.md5
# .$PLATFORM.mtd0.kwb.md5 will be appended to this
UBOOT_MTD0_BASE_MD5="https://raw.githubusercontent.com/mibodhi/kirkwood/kirkwood-2016-r1/checksum/uboot.bodhi.latest"

UBOOT_ENVIRONMENT_IMG_MD5="https://raw.githubusercontent.com/mibodhi/kirkwood/kirkwood-2016-r1/checksum/uboot.bodhi.latest.environment.img.md5"

UBOOT_MTD0="uboot.mtd0.kwb"
UBOOT_ENV="uboot.environment"




# Download locations
MIRROR="https://dl.dropboxusercontent.com/u/47541136/linux/tools_for_Kirkwood_installer" # from Alberto Bursi's dropbox
# full link https://github.com/mibodhi/kirkwood/blob/kirkwood-2016-r1/tools/busybox?raw=true
#from github
MIRROR_HEADER="https://github.com/mibodhi/kirkwood/blob/kirkwood-2016-r1/tools"
MIRROR_FOOTER="?raw=true"

# Where we download and keep tools temporarily
TOOLS="/tmp/tools"

# Tools and their download link
BLPARAM_URL="$MIRROR/blparam"
NANDDUMP_URL="$MIRROR_HEADER/nanddump$MIRROR_FOOTER"
NANDWRITE_URL="$MIRROR_HEADER/nandwrite$MIRROR_FOOTER"
FLASH_ERASE_URL="$MIRROR_HEADER/flash_erase$MIRROR_FOOTER"
FW_PRINTENV_URL="$MIRROR_HEADER/fw_printenv$MIRROR_FOOTER"
FW_CONFIG_URL="$MIRROR_HEADER/fw_env.config$MIRROR_FOOTER"

BUSYBOX_URL="$MIRROR_HEADER/busybox$MIRROR_FOOTER"
MKE2FS_URL="$MIRROR/mke2fs"
E2FSCK_URL="$MIRROR_HEADER/e2fsck$MIRROR_FOOTER"
SGDISK_URL="$MIRROR/sgdisk"

WGET_WITH_SSL_URL="http://pogoplug.s3.amazonaws.com/wget"

# MD5 urls
GENERIC_MD5_URL=$MIRROR #as there are no md5 anywhere for tools.
BLPARAM_MD5="$GENERIC_MD5_URL/blparam.md5"
NANDDUMP_MD5="$GENERIC_MD5_URL/nanddump.md5"
NANDWRITE_MD5="$GENERIC_MD5_URL/nandwrite.md5"
FLASH_ERASE_MD5="$GENERIC_MD5_URL/flash_erase.md5"
FW_PRINTENV_MD5="$GENERIC_MD5_URL/fw_printenv.md5"
#FW_SETENV_MD5="$GENERIC_MD5_URL/fw_setenv.md5" this uses the same binary as fw_printenv
FW_CONFIG_MD5="$GENERIC_MD5_URL/fw_env.config.md5"

BUSYBOX_MD5="$GENERIC_MD5_URL/busybox.md5"
MKE2FS_MD5="$GENERIC_MD5_URL/mke2fs.md5"
E2FSCK_MD5="$GENERIC_MD5_URL/e2fsck.md5"
SGDISK_MD5="$GENERIC_MD5_URL/sgdisk.md5"

#WGET_WITH_SSL_MD5="$GENERIC_MD5_URL/wget.md5" not used currently as we cannot download md5 for it

#uboot md5 used to check if current uBoot is recognized/valid or something, not used
#VALID_UBOOT_MD5="http://jeff.doozan.com/uboot/valid-uboot.md5"

# Tools and their place on disk
BLPARAM="$TOOLS/blparam"
NANDDUMP="$TOOLS/nanddump"
NANDWRITE="$TOOLS/nandwrite"
FLASH_ERASE="$TOOLS/flash_erase"
FW_PRINTENV="$TOOLS/fw_printenv"
FW_SETENV="$TOOLS/fw_setenv"
FW_CONFIG="$TOOLS/etc/fw_env.config"

BUSYBOX="$TOOLS/busybox"
MKE2FS="$TOOLS/mke2fs"
E2FSCK="$TOOLS/e2fsck"
SGDISK="$TOOLS/sgdisk"

WGET_WITH_SSL="$TOOLS/wget"

#filesystem to use when formatting
FILESYSTEM="ext3"

#default partition table to use
PARTITION_TABLE="MBR"

#####
#Device-specific definitions and commands are at the end of all this, in the MAIN, inside the "case" statement
#####


##############################
####
#### Some common utility functions used by the script's macrofunctions
####
##############################

verify_md5 ()
{
  file="$1"
  md5="$2"

  echo "verify file $file with md5 $md5"

  check_md5=$( cat "$md5" | cut -d' ' -f1)

  echo "$check_md5"

  file_md5=$(md5sum "$file" | cut -d' ' -f1)

  echo "$file_md5"

  if [ "$check_md5" = "$file_md5" ]; then
    echo "passed"
    return 0
  else
    echo "failed"
    return 1

  fi
}

download_and_verify () # add curl as a fallback     curl -O http://server1.cyberciti.biz/file.tar.gz
{
  file_dest="$1"
  file_url="$2"

  #now this comes from outside as we keep md5 on a different server
  md5_url="$3"

  md5_dest="$file_dest.md5"

  echo "$file_dest"
  echo "$file_url"
  echo "$md5_url"
  echo "$md5_dest"


  # Always download a fresh MD5, in case a newer version is available
  # the no-check-certificate in wget is needed because 99% of the cases the stock firmware will lack
  # a "legit certificates" list, so we must ask wget to not check the ssl certificate's validity.
  if [ -f "$md5_dest" ]; then rm -f "$md5_dest"; fi
  wget -O "$md5_dest" "$md5_url" --no-check-certificate
  # retry the download if it failed
  if [ ! -f "$md5_dest" ]; then
    wget -O  "$md5_dest" "$md5_url" --no-check-certificate
    if [ ! -f "$md5_dest" ]; then
      return 1 # Could not get md5
    fi
  fi

  # Download the file
  wget -O  "$file_dest" "$file_url" --no-check-certificate
  # retry the download if it failed
  verify_md5 "$file_dest" "$md5_dest"
  if [ "$?" != "0" ]; then
    echo "Download failed or MD5 did not match, try again"
    if [ -f "$file_dest" ]; then rm -f "$file_dest"; fi
    wget -O  "$file_dest" "$file_url" --no-check-certificate
    verify_md5 "$file_dest" "$md5_dest"
    if [ "$?" != "0" ]; then
      rm -f "$md5_dest"
      exit 1
    fi
  fi

  rm -f "$md5_dest"
  return 0
}

install ()
{
  file_dest="$1"
  file_url="$2"
  file_md5_url="$3" # md5 file url
  file_pmask="$4"  # Permissions mask

    echo
    echo "# Installing $file_dest..."

    if [ "$file_md5_url" = "NO_CHECK" ]; then
      # Download the file without checking md5
      wget -O  "$file_dest" "$file_url" --no-check-certificate
    fi

    if [ "$file_md5_url" != "NO_CHECK" ]; then
      download_and_verify "$file_dest" "$file_url" "$file_md5_url"
      if [ "$?" != "0" ]; then
        ERROR="## Could not install $file_dest from $file_url."
        error_harmless "$ERROR"
        exit 1
      fi
    fi

   echo " chmodding ""$file_pmask" " -- " "$file_dest"
    chmod "$file_pmask" "$file_dest"

    echo "# Successfully installed $file_dest."

  return 0
}

decide_what_drive_we_work_with(){

#since for various reasons I cannot get a RELIABLE way to find what device is friggin / I resort to ask the user

#looking at drives and detecting if sda sdb or mmcblk0 are there.
drive_sda=$(fdisk -l | grep /dev/sda | grep bytes )
drive_sdb=$(fdisk -l | grep /dev/sdb | grep bytes )
drive_mmcblk0=$(fdisk -l | grep /dev/mmcblk0 | grep bytes )
DISK=""

#deciding if there are no drives at all
if [ "$drive_sda" = "" ] && [ "$drive_mmcblk0" = "" ]; then
  echo "no disks detected, exiting now"
  exit 1
fi

#deciding if there is ONLY sda
if [ "$drive_sda" != "" ] && [ "$drive_sdb" = "" ]; then
  DISK="/dev/sda"
fi

#deciding if there is ONLY mmcblk0
if [ "$drive_mmcblk0" != "" ] && [ "$drive_sdb" = "" ]; then
  DISK="/dev/mmcblk0"
fi

#if after these checks DISK is still empty, we ask the user.
if [ "$DISK" = "" ]; then
  echo
  echo "what drive you want to install Debian in?"
  echo "(1,2,3,4,sd)"
  DISK=$(fdisk -l | grep /dev/sda | grep bytes |  awk '{if ($3 >= 1) print $1, $3, $4}')
  echo "test 1. $DISK"
  if [ "$DISK" != "" ]; then
    echo "1. $DISK"
  fi

  DISK=$(fdisk -l | grep /dev/sdb | grep bytes | awk '{if ($3 >= 1) print $1, $3, $4}' )
  if [ "$DISK" != "" ]; then
    echo "2. $DISK"
  fi

  DISK=$(fdisk -l | grep /dev/sdc | grep bytes | awk '{if ($3 >= 1) print $1, $3, $4}' )
  if [ "$DISK" != "" ]; then
    echo "3. $DISK"
  fi

  DISK=$(fdisk -l | grep /dev/sdd | grep bytes | awk '{if ($3 >= 1) print $1, $3, $4}' )
  if [ "$DISK" != "" ]; then
    echo "4. $DISK"
  fi

  DISK=$(fdisk -l | grep /dev/mmcblk0 | grep bytes | awk '{if ($3 >= 1) print $1, $3, $4}')
  if [ "$DISK" != "" ]; then
    echo "sd. $DISK"
  fi

read -r SELECTION

case $SELECTION in

    1)
      DISK="/dev/sda"
    ;;
    2)
      DISK="/dev/sdb"
    ;;
    3)
      DISK="/dev/sdc"
    ;;
    4)
      DISK="/dev/sdd"
    ;;
    sd)
      DISK="/dev/mmcblk0"
    ;;
  esac

fi

echo
echo "installation drive is $DISK"
echo

PART_1="${DISK}1"
PART_2="${DISK}2"

echo
echo "---unmounting installation drive"
echo

umount -l "$PART_1"
umount -l "$PART_2"


}

run_fw_printenv_setenv_in_chroot(){
command="$1"
args="$2"

#Surprise, surprise!
#In my Zyxel nsa310 can bind mount anything anywhere, but NOT /dev
#so making a chroot is pointless as it won't see /dev/mtd0
#I resort to a simpler hack I already did in the past
#commenting out the chrooting that may be useful for the future
#theoretically I can use makedevs to create other device files in the new dev

#preparing folder mounts for chroot

# mkdir -p "$TOOLS/proc"
# mount proc "$TOOLS/proc" -t proc
#
# mkdir -p "$TOOLS/sysfs"
# mount proc "$TOOLS/sysfs" -t sysfs
#
# mkdir -p "$TOOLS/dev"
# mount -o bind "/dev" "$TOOLS/dev"
#
# #chrooting to give the command only
#
# chroot "$TOOLS"/ ./"$command" "$args"
#
# #unmounting stuff
# umount "$TOOLS/proc"
# umount "$TOOLS/sysfs"
# umount "$TOOLS/dev"


#copying /etc to another folder
mkdir -p "$TOOLS/etc_original"
cp -rf "/etc/." "$TOOLS/etc_original"
#making a tmpfs over the damn /etc
mount -t tmpfs -o rw tmpfs "/etc"
#copying the /etc contents into it,
cp -rf "$TOOLS/etc_original/." "/etc/"
#changing the configs I need to change and then run the command.
cp -rf "$TOOLS/etc/." "/etc/"

"$command" "$args"

#cool, we now remove the tmpfs
umount "/etc"

#and dump the clone of stock /etc
rm -r "$TOOLS/etc_original"

}

read_env_stock_uboot(){

#reads a stock env and returns its value, using various ways

ENV_NAME=$1

#cleaning the variable
VALUE=""

#calling stock blparam

#exporting original path so we use tools from stock system
export PATH="$ORIGINAL_PATH"


#if ENV_NAME is empty we don't filter output as we need a full dump
if [ "$ENV_NAME" = ""  ]; then
  #if there is blparam in stock fw we use it
  echo "trying to read the env with stock blparam"
  VALUE=$(blparam)

  if [ "$?" != "0" ]; then
    printf "%s" "$VALUE"
    echo "if there is fw_prinenv in stock fw we use it"
    VALUE=$(fw_printenv)
  fi

  if [ "$?" != "0" ]; then
    printf "%s" "$VALUE"
    #exporting our own tools path so we use tools from our toolbox again
    export PATH="$TOOLS"
    echo "we try with our own fw_printenv, hoping there is a config in the right place"
    VALUE=$(fw_printenv)
  fi

else
#ENV_NAME isn't empty so we use it to filter the output

  echo "trying to read the env with stock blparam"
  VALUE=$(blparam | grep "^$ENV_NAME=" | sed "s@$ENV_NAME=@@g"  )

  if [ "$VALUE" = "" ]; then
    printf "%s" "$VALUE"
    echo "we try reading envs by using stock fw_printenv"
    VALUE=$(fw_printenv "$ENV_NAME" | sed "s@$ENV_NAME=@@g" )

  fi

  if [ "$VALUE" = "" ]; then
    printf "%s" "$VALUE"
    echo "we try with our own fw_printenv, hoping there is a config in the right place"
    VALUE=$(fw_printenv "$ENV_NAME" | sed "s@$ENV_NAME=@@g" )
  fi
fi

if [ "$VALUE" = "" ]; then
  printf "%s" "$VALUE"
  echo "the above command failed again, we return an error code"
  export PATH="$TOOLS"
  return 1
fi

printf "%s" "$VALUE"
#exporting our own tools path so we use tools from our toolbox again, just in case
export PATH="$TOOLS"

}

read_env_modern_uboot(){

#reads a modern uboot env and returns its value, using various ways

ENV_NAME=$1

#cleaning the variable
VALUE=""

#if ENV_NAME is empty we don't filter output as we need a full dump
echo "we now try reading the envs from an older version of our own uboot envs"

if [ "$ENV_NAME" = ""  ]; then
     VALUE=$(run_fw_printenv_setenv_in_chroot "fw_printenv")
else
     VALUE=$(run_fw_printenv_setenv_in_chroot "fw_printenv" "$ENV_NAME")
fi

if [ "$?" != "0" ]; then
  printf "%s" "$VALUE"
  echo "the above command failed again, we return an error code"
  return 1
fi

printf "%s" "$VALUE"
}

##############################
####
#### functions doing a step in the install process
####
##############################

create_tmpfs(){
#we generate our own tmpfs, then we mount it as our own tool directory.
mkdir -p $TOOLS

#unmounting any previous mounting of this
umount -l $TOOLS

mount -t tmpfs -o rw,dev,exec,suid tmpfs $TOOLS

if [ "$?" != "0" ]; then
  ERROR="## Could not make the tmpfs to install tools in."
  error_harmless "$ERROR"
  exit 1
fi

}

download_tools() {

#creating the tmpfs to download tools into
create_tmpfs

# Create the tools and TOOLS/etc directory if it doesn't already exist
mkdir -p "$TOOLS/etc"

#downloading manually a wget that can download from https links
wget -O  "$WGET_WITH_SSL"           "$WGET_WITH_SSL_URL"
chmod "755" "$WGET_WITH_SSL"

#downloading manually busybox with the new wget, then manually checking
"$WGET_WITH_SSL" -O  "$BUSYBOX"       "$BUSYBOX_URL" --no-check-certificate
"$WGET_WITH_SSL" -O  "$BUSYBOX.md5"       "$BUSYBOX_MD5" --no-check-certificate
verify_md5 "$BUSYBOX" "$BUSYBOX.md5"
  if [ "$?" != "0" ]; then
    echo "Download failed or MD5 did not match, try again"
    if [ -f "$BUSYBOX" ]; then rm -f "$BUSYBOX"; fi
    wget -O  "$BUSYBOX" "$BUSYBOX_URL" --no-check-certificate
    verify_md5 "$BUSYBOX" "$BUSYBOX.md5"
    if [ "$?" != "0" ]; then
      ERROR="## Could not verify busybox download."
      error_harmless "$ERROR"
      exit 1
    fi
  fi

chmod "755" "$BUSYBOX"

#backing up the standard PATH so we can use it later too.
ORIGINAL_PATH="$PATH"

#changing PATH so we call stuff from our own folders from now on
#placing it before the stock path so all commands will be first called from here
#first goes the tools directory so we should call the right wget
export PATH="$TOOLS"

echo "new PATH is now $PATH"

#adding symlinks to all the tools

#moving into the folder.
cd $TOOLS || exit 1
$BUSYBOX ln -s $BUSYBOX ln #first is busybox's own symlink making a symlink for itself, then we go on

for f in  [ [[ acpid addgroup adduser adjtimex ar arping ash awk basename blockdev brctl bunzip2 bzcat bzip2 cal cat chgrp chmod chown chroot chvt clear cmp cp cpio crond crontab cut date dc dd deallocvt delgroup deluser df dirname dmesg dnsdomainname dos2unix dpkg dpkg-deb du dumpkmap dumpleases echo ed egrep eject env expand expr false fbset fdflush fdisk fgrep find fold free freeramdisk fsck.minix ftpget ftpput getopt getty grep gunzip gzip halt head hexdump hostid hostname httpd hwclock id ifconfig ifdown ifup init ionice ip ipcalc kill killall klogd last length less linuxrc loadfont loadkmap logger login logname logread losetup ls lzcat makedevs md5sum mdev mesg microcom mkdir mkfifo mkfs.minix mknod mkswap mktemp more mount mt mv nameif nc netstat nslookup od openvt passwd patch pidof ping ping6 pivot_root poweroff printf ps pwd rdate readlink realpath reboot renice reset rm rmdir route rpm rpm2cpio run-parts sed setkeycodes sh sha1sum sha256sum sha512sum sleep sort start-stop-daemon strings stty su sulogin swapoff swapon sync syslogd tac tail tar tee telnet telnetd test tftp time timeout top touch tr traceroute traceroute6 true tty tunctl udhcpc udhcpd umount uname uncompress unexpand uniq unix2dos unlzma unzip uptime usleep uudecode uuencode vconfig vi vlock watch watchdog wc which who whoami xargs yes zcat ; do

ln -s $BUSYBOX "$f"
done

#ln -s $BUSYBOX  wget # we use ours here
#whew, done.

chmod -R "755" $TOOLS

ls -l

cd || exit 1
}

install_uboot_tools(){
#keeping uboot tools separated so we can use as little tmpfs/ram space as possible

install "$BLPARAM"          "$BLPARAM_URL"      "$BLPARAM_MD5"         755
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify blparam download."
      error_harmless "$ERROR"
      exit 1
    fi
install "$NANDWRITE"        "$NANDWRITE_URL"    "$NANDWRITE_MD5"       755
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify nandwrite download."
      error_harmless "$ERROR"
      exit 1
    fi
install "$NANDDUMP"         "$NANDDUMP_URL"     "$NANDDUMP_MD5"        755
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify nanddump download."
      error_harmless "$ERROR"
      exit 1
    fi
install "$FLASH_ERASE"      "$FLASH_ERASE_URL"  "$FLASH_ERASE_MD5"     755
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify flash_erase download."
      error_harmless "$ERROR"
      exit 1
    fi
install "$FW_PRINTENV"      "$FW_PRINTENV_URL"  "$FW_PRINTENV_MD5"     755
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify fw_printenv download."
      error_harmless "$ERROR"
      exit 1
    fi

if [ ! -f "$FW_SETENV" ]; then
  ln -s "$FW_PRINTENV" "$FW_SETENV" 2> /dev/null
fi

install "$FW_CONFIG"        "$FW_CONFIG_URL"    "$FW_CONFIG_MD5"       644
    if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify fw_config download."
      error_harmless "$ERROR"
      exit 1
    fi

}

remove_uboot_tools(){

rm "$BLPARAM"
rm "$NANDWRITE"
rm "$NANDDUMP"
rm "$FLASH_ERASE"
rm "$FW_PRINTENV"
rm "$FW_SETENV"
rm -r "$FW_CONFIG"

}

install_formatting_tools(){

#keeping format tools separated so we can use as little tmpfs/ram space as possible

#install "$SGDISK"           "$SGDISK_URL"       "$SGDISK_MD5"          755 disabled until I recompile a sgdisk that works
install "$E2FSCK"           "$E2FSCK_URL"       "$E2FSCK_MD5"          755
if [ "$?" != "0" ]; then
    ERROR="## Could not download or verify e2fsck download."
    error_harmless "$ERROR"
    exit 1
fi

install "$MKE2FS"           "$MKE2FS_URL"       "$MKE2FS_MD5"          755
if [ "$?" != "0" ]; then
    ERROR="## Could not download or verify mke2fs download."
    error_harmless "$ERROR"
    exit 1
fi

}

remove_formatting_tools(){

rm "$E2FSCK"
rm "$MKE2FS"

}

check_bad_blocks(){
# checking if there are bad blocks

# Example of bad block in mtd0:
#
# dmesg | grep -i 'bad'
#
# [    2.413231] Scanning device for bad blocks
# [    2.417731] Bad eraseblock 3 at 0x000000060000
#
#
# This bad block is the 4th block which resides within mtd0 (block 0 to block 7). So u-boot should not be flashed.

#we do a loop to check that no block in the first MB or so is good, we don't care about the rest.

#128k = 131072 bytes -> 0x20000 in hexadecimal and we sum 0x20000 to it to get our addresses.

for f in 00000 20000 40000 60000 80000 a0000 c0000 e0000 ; do

# call dmesg, show only lines with "Bad eraseblock" lower or uppercase,
#remove "0x" and any trailing zeroes so we get only the actual number of the hex address
#then finally check if in these lines there is the address we are looking for
dmesg | grep -i 'Bad eraseblock' | sed 's/0x[0-0][0-0]*//' | grep -i $f

if [ "$?" = "0" ]; then
    ERROR="## Bad blocks in 0x$f area on NAND, uboot must be installed manually."
    error_harmless "$ERROR"
    exit 1
fi

done

}

#need to make a fallback, so that if flashing uboot fails somehow we reflash the stock uboot
install_uboot() {

# Install uBoot on /dev/mtd0


#hardcoding this to trigger the if below
UPDATE_UBOOT=1

# Download and install the latest uBoot
if [ "$UPDATE_UBOOT" = "1" ]; then

  echo ""
  echo "# Installing uBoot"

  #generating uboot MD5 link
  UBOOT_MD5_URL="$UBOOT_MTD0_BASE_MD5.$PLATFORM.mtd0.kwb.md5"

  echo "## Installing u-boot-bodhi-latest.$PLATFORM.mtd0.kwb"

  rm -f "$TOOLS/$UBOOT_MTD0"

  download_and_verify "$TOOLS/$UBOOT_MTD0" "$UBOOT_URL" "$UBOOT_MD5_URL"
  if [ "$?" != "0" ]; then
      ERROR="## Could not download or verify new uBoot download."
      error_harmless "$ERROR"
      exit 1
  fi

  # hardcoding this for now, having this as a variable is more flexible
  # like for example in case we are installing from Debian
  DEV_MTD0="$STOCK_MTD0"

  #downloading uboot md5 to use for uboot verification BEFORE writing uboot
  #so if this fails we can still call an error_harmless and everything will be fine
  if [ -f "$TOOLS/new_uboot_dump.md5" ]; then rm "$TOOLS/new_uboot_dump.md5"; fi
  wget -O "$TOOLS/new_uboot_dump.md5" "$UBOOT_MD5_URL" --no-check-certificate
    if [ ! -f "$TOOLS/new_uboot_dump.md5" ]; then
    wget -O  "$TOOLS/new_uboot_dump.md5" "$UBOOT_MD5_URL" --no-check-certificate
    if [ ! -f "$TOOLS/new_uboot_dump.md5" ]; then
      ERROR="## Could not download the md5 to verify the uBoot."
      error_harmless "$ERROR"
      exit 1
    fi
  fi

  # Write new uBoot to mtd0
  # Erase the first 512k
  flash_erase "$DEV_MTD0" 0 4
  #need to add a check on the output here.

  nandwrite "$DEV_MTD0" "$TOOLS/$UBOOT_MTD0"
  #need to add a check on the output here.


  # dump mtd0 and compare the checksum, to make sure it installed properly
  nanddump --noecc --omitoob -l 0x80000 -f "$TOOLS/new_uboot_dump" "$DEV_MTD0"
  echo "## Verifying new uBoot..."

  #verification to see if flash went well
  verify_md5 "$TOOLS/new_uboot_dump" "$TOOLS/new_uboot_dump.md5"
  if [ "$?" != "0" ]; then
    rm -f "$TOOLS/new_uboot_dump.md5"

    ERROR="## uBoot was not properly installed to mtd0."
    SECOND_MESSAGE="## Make a backup of the content of $BACKUP_DIR in the installation drive someplace safe"
    error_dont_reboot "$ERROR" "$SECOND_MESSAGE"
    exit 1
  else
    echo "# Verified successfully!"
  fi
  rm -f "$TOOLS/new_uboot_dump.md5"

fi

}

install_envs() {
#now receives the results of the checks to see if we need to update envs too

#bunch of code that we don't need IF or until we enable uboot md5 checking

#if [ "$UPDATE_UBOOT" = "0" ]; then
#  UPDATE_UBOOT_ENVIRONMENT=0
#elif [ "$UBOOT_VERSION" = "original" -o "$UBOOT_VERSION" = "unknown" ]; then
#  UPDATE_UBOOT_ENVIRONMENT=1
#else  # Updating uboot, but it's from a known "new" uboot, no need to reset the environment
#  UPDATE_UBOOT_ENVIRONMENT=0
#fi

#if [ "$UPDATE_UBOOT_ENVIRONMENT" = "0" -a "$1" ]; then
#  echo ""
#  echo ""
#  if [ "$UPDATE_UBOOT" = "0" ]; then
#    echo "You are already running the latest uBoot."
#  else
#    echo "You are already running a modern uBoot."
#  fi
#  echo "Your current uBoot environment should be reasonable.  However, if you're having"
#  echo "any probems booting, you can reset the environment variables to known good values."
#  echo -n "Would you like to reset the uBoot environment? [N/y] "
#  read PROMPT
#  if [ "$PROMPT" = "y" -o "$PROMPT" = "Y" ]; then
#    UPDATE_UBOOT_ENVIRONMENT=1
#  fi
#fi

echo "We are now ready to flash uboot environment, its boot configuration."
echo ""
echo "If this is the first time you run this script, answer Y"
echo ""
echo "If you are updating a previous version of bodhi's uboot and have"
echo "custom uboot envs you wish to keep, answer N"
echo ""
echo ""
echo "Would you like to install the new uboot environment? [Y/n] "
read -r PROMPT
  if [ "$PROMPT" = "n" ] || [ "$PROMPT" = "N" ]; then
    UPDATE_UBOOT_ENVIRONMENT="no"
  fi

if [ "$UPDATE_UBOOT_ENVIRONMENT" != "no" ]; then
  echo ""
  echo "# Installing uBoot environment"

  # Install the uBoot environment
  download_and_verify "$TOOLS/$UBOOT_ENV" "$UBOOT_ENVIRONMENT_IMG_URL" "$UBOOT_ENVIRONMENT_IMG_MD5"
  if [ "$?" != "0" ]; then
    #this is a "error_harmless" because we flash uboot envs BEFORE uboot, so if download fails no damage is done
    ERROR="## Could not download uBoot environment"
    error_harmless "$ERROR"
    exit 1
  fi

  #checking if we can download the md5 file, BEFORE flashing anything, so if this fails we can
  # throw an error_harmless and it will be fine
  wget -O "$TOOLS/$UBOOT_ENV.md5" "$UBOOT_ENVIRONMENT_IMG_MD5" --no-check-certificate
    if [ ! -f "$TOOLS/$UBOOT_ENV.md5" ]; then
      wget -O "$TOOLS/$UBOOT_ENV.md5" "$UBOOT_ENVIRONMENT_IMG_MD5" --no-check-certificate
      if [ ! -f "$TOOLS/new_uboot_dump.md5" ]; then
        ERROR="## Could not download the md5 to verify the uBoot environment."
        error_harmless "$ERROR"
        exit 1
      fi
    fi

  #flashing uboot envs
  flash_erase "$DEV_MTD0" 0xc0000 1
  nandwrite "-s" "786432" "$DEV_MTD0" "$TOOLS/$UBOOT_ENV"

  echo ""
  echo "# Verifying uBoot environment"

  # Verify the uBoot environment
  nanddump --noecc --omitoob -f "$TOOLS/uboot.environment.dump" -s 0xc0000 -l 0x20000 "$DEV_MTD0"

  verify_md5 "$TOOLS/uboot.environment.dump" "$TOOLS/$UBOOT_ENV.md5"
  if [ "$?" != "0" ]; then
    rm -f "$TOOLS/$UBOOT_ENV.md5"
    ERROR="## uBoot environment was not properly written to mtd0."
    SECOND_MESSAGE="## Make a backup of the content of $BACKUP_DIR in the installation drive someplace safe"
    error_dont_reboot "$ERROR" "$SECOND_MESSAGE"
    exit 1
  fi

  rm -f "$TOOLS/$UBOOT_ENV.md5"
  rm -f "$TOOLS/uboot.environment_dump"

fi

echo ""
echo "# uBoot environment installation has completed successfully."

}

preserve_crucial_envs_for_later() {

#preserving crucial envs so we can add them again later

  # Preserve the MAC address

  #read from stock uboot
  read_env_stock_uboot "ethaddr"
  if [ "$?" != "0" ] || [ $VALUE = "" ]; then
    #read from modern uboot
    read_env_modern_uboot "ethaddr"
    if [ "$?" != "0" ] || [ $VALUE = "" ]; then
      #if it still fails we try to get the info to fill this variable through other ways
      ENV_ETHADDR=$(ifconfig -a | grep eth0 | awk '{ print $5 }')
      if [ "$ENV_ETHADDR" = "" ] ; then
        #still empty, aborting
        ERROR="## Could not read ethaddr uboot variable."
        error_harmless "$ERROR"
        exit 1
      fi
    fi
  ENV_ETHADDR="$VALUE"


  fi

  # Preserve the MAC address of the second ethernet port (if it's there)
  #read from stock uboot
  read_env_stock_uboot "eth1addr"
  if [ "$?" != "0" ] || [ $VALUE = "" ]; then
    #read from modern uboot
    read_env_modern_uboot "eth1addr"
    if [ "$?" != "0" ] || [ $VALUE = "" ]; then
      #if it still fails we try to get the info to fill this variable through other ways
      ENV_ETH1ADDR=$(ifconfig -a | grep eth1 | awk '{ print $5 }')
      if [ "$ENV_ETH1ADDR" = "" ] ; then
        #still empty, since having two eth is uncommon in kirkwood boxes,
        #I assume this isn't an error and the box really does not have eth1.
        #so I clean the VALUE so we are sure the variable is set as empty later.
        VALUE=""
      fi
    fi
  ENV_ETH1ADDR="$VALUE"


  fi

  # Preserve the mtdparts
  read_env_stock_uboot "mtdparts"
  if [ "$?" != "0" ] || [ $VALUE = "" ]; then
    #if the above command failed, we proceed to acquire this info in other ways still from stock uboot

    #usually the mtdparts is in the bootargs variable, so we try that
    read_env_stock_uboot "bootargs"
    echo "read bootargs as $VALUE"

    #now we extract the mtdparts from that line
    #and we also drop the "mtdparts=nand_mtd:" part as we need that to be "mtdparts=orion_nand:"
    MTDPARTS=$( printf "%s" "$VALUE" | awk '{ print $1, "\n" $2, "\n" $3, "\n" $4, "\n" $5, "\n" }' | grep "mtdparts" | awk -F  ":" '{print $2}')

    echo "MTDPARTS is $MTDPARTS"

    #if it is empty the above has failed so we try another thing
    if [ "$MTDPARTS" = "" ] ; then
    # we read it from modern uboot
      read_env_modern_uboot "mtdparts"
      if [ "$?" != "0" ] || [ $VALUE = "" ]; then
        #still failed or empty, aborting
        ERROR="## Could not read mtdparts uboot variable."
        error_harmless "$ERROR"
        exit 1
      fi
      echo "MTDPARTS is $MTDPARTS"
      #still removing the "mtdparts=nand_mtd:" part
      MTDPARTS=$(printf "%s" "$VALUE" | grep "mtdparts" | awk -F  ":" '{print $2}')
    fi
  fi

  #if the MTDPARTS is still empty it means the value in $VALUE is correct and we skipped the above checks,
  #therefore we load it still dropping the part we don't want
  if [ "$MTDPARTS" = "" ] ; then
  MTDPARTS=$(printf "%s" "$VALUE" | grep "mtdparts" | awk -F  ":" '{print $2}')
  fi

  #cool beans, if we got this far we should have a MTDPARTS loaded
  #therefore we add the "mtdparts=orion_nand:" and generate the actual env value

  ENV_MTDPARTS="mtdparts=orion_nand:$VALUE"

}

backing_up_uboot_and_envs(){

#backing up stock envs and stock uboot

#if the disk to work with isn't defined, we run the function to decide what drive we work with
if [ "$DISK" = "" ]; then
  decide_what_drive_we_work_with
fi

#mounting rootfs partition
mkdir -p "/tmp/rootfs"

echo
echo "partition to mount is $PART_1"

mount "$PART_1" "/tmp/rootfs"

BACKUP_DIR="/tmp/rootfs/uboot_backups"
mkdir -p "$BACKUP_DIR"

#name of uboot backup
UBOOT_ORIGINAL="uboot-$PLATFORM-mtd0.kwb"

#assigning MTD0 at DEV_MTD0
DEV_MTD0="$STOCK_MTD0"

#defining current date
DATE="$(date +%F_%R)"

#dumping all envs from stock or modern uboot
read_env_stock_uboot
if [ "$?" = "0" ]; then
  printf "%s" "$VALUE" > "$TOOLS/stock_env_backup.txt"
  mv "$TOOLS/stock_env_backup.txt" "$BACKUP_DIR/stock_env_backup-$DATE.txt"
fi

read_env_modern_uboot
if [ "$?" = "0" ]; then
  printf "%s" "$VALUE" > "$TOOLS/modern_env_backup.txt"
  mv "$TOOLS/modern_env_backup.txt" "$BACKUP_DIR/modern_env_backup-$DATE.txt"
fi

echo "reading stock_env_backup.txt"
cat "$BACKUP_DIR/stock_env_backup$DATE.txt"

echo "reading modern_env_backup.txt"
cat "$BACKUP_DIR/modern_env_backup$DATE.txt"

# dump mtd0
nanddump --noecc --omitoob -l 0x80000 -f "$TOOLS/$UBOOT_ORIGINAL" "$DEV_MTD0"
mv "$TOOLS/$UBOOT_ORIGINAL" "$BACKUP_DIR/$UBOOT_ORIGINAL-$DATE"

}

set_crucial_envs(){
#writing all envs that MUST be set for the box to boot/work at all

#running all commands inside a chroot so we call the same function making/removing the chroot

# writing MAC back
run_fw_printenv_setenv_in_chroot "fw_setenv ethaddr $ENV_ETHADDR"
if [ "$?" != "0" ]; then
    ERROR="## Could not set ethaddr uboot env."
    error_harmless "$ERROR"
    exit 1
  fi

if [ "$ENV_ETH1ADDR" != "" ] ; then
      # writing MAC of second port back
      run_fw_printenv_setenv_in_chroot "fw_setenv eth1addr $ENV_ETH1ADDR"
      if [ "$?" != "0" ]; then
        ERROR="## Could not set eth1addr uboot env."
        error_harmless "$ERROR"
        exit 1
      fi
fi

# writing mtdparts back
run_fw_printenv_setenv_in_chroot "fw_setenv mtdparts $ENV_MTDPARTS"
if [ "$?" != "0" ]; then
    ERROR="## Could not set mtdparts uboot env."
    error_harmless "$ERROR"
    exit 1
fi

# adding stuff that was not there in stock uboot but we need
# writing dtb
run_fw_printenv_setenv_in_chroot "fw_setenv dtb_file $ENV_DTB_FILE"
if [ "$?" != "0" ]; then
    ERROR="## Could not set dtb_file uboot env."
    error_harmless "$ERROR"
    exit 1
fi

# writing a workaround to make sure boot from Sata works fine
# This will help the "ide reset" to work properly.
# There seems to be a bug in u-boot that if you do "ide reset" too quickly in succession,
# the SATA drive might have problem spinning up
BOOTCMD_UENV_FIX="run uenv_load; if test $uenv_loaded -eq 1; then run uenv_import; fi; sleep 3"

run_fw_printenv_setenv_in_chroot "fw_setenv bootcmd_uenv $BOOTCMD_UENV_FIX"
if [ "$?" != "0" ]; then
    ERROR="## Could not set bootcmd_uenv_sata_fix uboot env."
    error_harmless "$ERROR"
    exit 1
fi

# writing arcNumber and machid back, if any, otherwise this same code will clear the variables with same name already there
run_fw_printenv_setenv_in_chroot "fw_setenv arcNumber $ARCNUMBER"
if [ "$?" != "0" ]; then
    ERROR="## Could not set arcNumber uboot env."
    error_harmless "$ERROR"
    exit 1
fi
run_fw_printenv_setenv_in_chroot "fw_setenv machid $MACHID"
if [ "$?" != "0" ]; then
    ERROR="## Could not set machid uboot env."
    error_harmless "$ERROR"
    exit 1
fi

echo "now the full envs of new uboot are:"

run_fw_printenv_setenv_in_chroot "fw_printenv"


#writing custom boot params (mostly systemd)
if [ "$CUSTOM_PARAM" != "" ]; then
  run_fw_printenv_setenv_in_chroot "fw_setenv custom_params $CUSTOM_PARAM"
  if [ "$?" != "0" ]; then
    ERROR="## Could not set custom_params uboot env."
    error_harmless "$ERROR"
    exit 1
  fi
fi
}

partition_and_format_disk() {

#if the disk to work with isn't defined, we run the function to decide what drive we work with
if [ "$DISK" = "" ]; then
  decide_what_drive_we_work_with
fi


  install_drive_size_unit=$(fdisk -l | grep $DISK | grep bytes | awk '{if ($3 >= 1) print $4;}')

echo
echo "---unmounting installation drive"
echo

umount -l "$PART_1"
umount -l "$PART_2"

echo
echo "---partitioning installation drive"
echo

#checking install drive size
install_drive_size=$(fdisk -l | grep $DISK | grep bytes | awk '{print $3}')

echo "install_drive_size is $install_drive_size"

if [ "$install_drive_size_unit" = "MB," ]; then
  #so the drive is detected in MB, we sort it depending on size

  # turning the size value into an integer
  install_drive_size_integer=$( echo "$install_drive_size" | sed 's/\..*//' )

  echo "install_drive_size_integer is $install_drive_size_integer"

  #if storage device is smaller than 1 GB the variable should remain empty
  if (( "$install_drive_size_integer" < 1024 )); then
    echo "the new rootfs drive size is less than 1 GB."
    echo "It is a bit too small for confort and will have subpar performance,"
    echo "aborting installation now."

    exit 1
  fi

  #doing a bunch of checks to decide what partition layout to create.

  if (( "$install_drive_size_integer" < 33000 )); then

    echo "drive is smaller than 33GB so we assume this is a flash drive."
    echo "we will create only a system partition, no swap nor data partition."

    fdisk "$DISK" <<EOF
o
n
p
1


w
EOF

  else

  echo "drive is bigger than 33 GB so we assume this is a hard drive."
  echo "making a 7 GB system partition, a 1 GB swap partition and"
  echo "the rest of the space becomes a data partition"

  NEED_DATA="yes"

  fdisk "$DISK" <<EOF
o
n
p
1

+7168M
n
p
2

+1024M
n
p
3


w
EOF

fi

fi

if [ "$install_drive_size_unit" = "GB," ]; then
#if drive is detected in GB unit we assume it is a hard drive

  NEED_DATA="yes"

  echo "we assume this is a hard drive."
  echo "making a 6 GB system partition and"
  echo "the rest of the space becomes a data partition"

  fdisk "$DISK" <<EOF
o
n
p
1

+7168M
n
p
2


w
EOF

fi

fdisk -l

echo "---pausing for a bit so ioctl can re-sync partitions"
sleep 10

#  make filesystems
echo "---formatting the partitions we created"

#we need to unmount aggressively as the stock firmware mounts stuff aggressively
umount -l "$PART_1"
umount -l "$PART_2"

if [ -f "/sbin/e2label" ]; then
# this means we are inside a Debian rootfs,
# labelling it now as rootfs will screw up mounting later
# it will be labelled later when unmounted
mke2fs -t "$FILESYSTEM" "$PART_1"
else
#not in Debian rootfs, so we can give it a label only this way
mke2fs -t "$FILESYSTEM" -L "rootfs" "$PART_1"
fi

#checking if swap and data partition is needed
if [ "$NEED_DATA" = "yes" ]; then
mke2fs -t "ext4" -L "data" "$PART_2"
fi

umount -l "$PART_1"
umount -l "$PART_2"


echo '---doing file system check on partitons to be sure that they are ok'
e2fsck -fp "$PART_1"
if [ "$NEED_DATA" = "yes" ]; then
e2fsck -fp "$PART_2"
fi
} # end of partition_and_format_disk


download_and_write_rootfs(){

#if the disk to work with isn't defined, we run the function to decide what drive we work with
if [ "$DISK" = "" ]; then
  decide_what_drive_we_work_with
fi

echo
echo "---unmounting installation drive"
echo

umount -l "$PART_1"
umount -l "$PART_2"


###
#  make dirs & mount
echo "---mounting partitions"
mkdir -p "/tmp/rootfs"

echo
echo "partition to mount is $PART_1"
sleep 5

mount "$PART_1" "/tmp/rootfs"

echo "mounted partitions are"
mount
echo
echo

echo "---checking that the stuff was mounted and writable"
# Just because it's better to.
echo "testing if this is readable" > "/tmp/rootfs/testfile"
if [ ! -f "/tmp/rootfs/testfile" ]; then
    echo "Rootfs partition not created! Aborting!"
    exit 1
fi

#removing the test file
rm "/tmp/rootfs/testfile"
echo "---everything writable"
echo

#downloading rootfs in the rootfs partition, aborting if fails the md5 check.

download_and_verify "$ROOTFS" "$ROOTFS_URL" "$ROOTFS_MD5"
if [ "$?" -ne "0" ]; then
        echo "## Could not install $ROOTFS from $ROOTFS_URL, exiting."
        exit 1
      fi

#  extract the rootfs
echo "---extracting debian rootfs to install Debian"
cd "/tmp/rootfs/" || exit 1
bunzip2 "$ROOTFS"
tar xvf "$ROOTFS_TAR"
rm "$ROOTFS_TAR"
cd || exit 1
#  always make sure you're sync'd!fi
sync

#checking to see if it went right
if [ ! -f "/tmp/rootfs/etc/fstab" ]; then
    echo "root files not extracted correctly! Aborting!"
    exit 1
fi
echo "---everything fine"

if [ -f "/sbin/e2label" ]; then # this means we are inside a Debian rootfs
  umount "$PART_1"
  /sbin/e2label "$PART_1" "rootfs"
fi

umount -l "$PART_1"
umount -l "$PART_2"

}

gather_telemetry(){
#dumping a bunch of data about the stock firmware and box

#if the platform isn't set we ask what device we are working on
if [ "$PLATFORM" = "" ]; then
echo "please state what device we are analyzing"
read -r PLATFORM
fi

TELEMETRY_FOLDER="$TOOLS/telemetry"
mkdir -p "$TELEMETRY_FOLDER"

#gathering info about stock firmware
TELEMETRY_TEXT="$TELEMETRY_FOLDER/telemetry-$PLATFORM.txt"

for f in "ls -lR" "cat /proc/cpuinfo" "cat /proc/meminfo" "cat /proc/version" "cat /proc/mtd" "cat /proc/cmdline" "cat /proc/swaps" "cat /proc/mdstat" "cat /proc/filesystems" "cat /proc/modules" "df" "mount" "/bin/busybox" "/lib/libc.so.6" "dmesg" "free" "ifconfig" "ps aux" "ps axjf" "blparam" "fw_printenv" ; do

  echo "" >> "$TELEMETRY_TEXT"
  echo "-----------------------------------" >> "$TELEMETRY_TEXT"
  echo "output of $f" >> "$TELEMETRY_TEXT"
  $f 2> /dev/null >> "$TELEMETRY_TEXT"

done

#gathering info using firmware's own tools
  export PATH="$ORIGINAL_PATH"

for f in "blparam" "fw_printenv" ; do

  echo "" >> "$TELEMETRY_TEXT"
  echo "-----------------------------------" >> "$TELEMETRY_TEXT"
  echo "output of stock firmware $f" >> "$TELEMETRY_TEXT"
  $f 2> /dev/null >> "$TELEMETRY_TEXT"

done

#ok done we return to use our own tools
export PATH="$TOOLS"

#dumping the mtd parts that appear to be carrying uboot or envs
#giving the dump the same name it had in stock mtdparts
for f in "1" "2" "3" "4" "5" "6" "7" "8" ; do

  MTD_TO_DUMP=$(cat "/proc/mtd" | grep boot | sed -n "$f"p | awk '{print $1}' | sed 's/.$//' )
  MTD_NAME=$(cat "/proc/mtd" | grep boot | sed -n "$f"p | awk '{print $4}' |  sed 's/"//g' )
  if [ "$MTD_TO_DUMP" != "" ]; then
    nanddump --noecc --omitoob -f "$TELEMETRY_FOLDER/$MTD_NAME-$PLATFORM-$MTD_TO_DUMP" "/dev/$MTD_TO_DUMP"
  fi

done

#setting the date
DATE="$(date +%F_%R)"

#setting the archive name
ARCHIVE_NAME="$PLATFORM-telemetry-$DATE.tar"

#bundling stuff, ready for uploading
cd $TOOLS || return 1
tar -cvf "$ARCHIVE_NAME" "$TELEMETRY_FOLDER"
bzip2 "$ARCHIVE_NAME"

#uploading to ftp server
#ftpput -v -u "anonymous" -p "" "alby.ftp.sh" "/uploads/$ARCHIVE_NAME.bz2" "$ARCHIVE_NAME.bz2"

cd || return 1

#ftp does not seem to work reliably, so we offer to save also locally the file.
#if the disk to work with isn't defined, we run the function to decide what drive we work with
  if [ "$DISK" = "" ]; then
    decide_what_drive_we_work_with
  fi
echo
echo "---unmounting installation drive"
echo

umount -l "$PART_1"
umount -l "$PART_2"


###
#  make dirs & mount
echo "---mounting partitions"
mkdir -p "/tmp/rootfs"

echo
echo "partition to mount is $PART_1"
sleep 5

mount "$PART_1" "/tmp/rootfs"

echo "mounted partitions are"
mount
echo
echo

echo "---checking that the stuff was mounted and writable"
# Just because it's better to.
echo "testing if this is readable" > "/tmp/rootfs/testfile"
if [ ! -f "/tmp/rootfs/testfile" ]; then
    echo "Rootfs partition not created! Aborting!"
    exit 1
fi

#removing the test file
rm "/tmp/rootfs/testfile"
echo "---everything writable"
echo

#moving the telemetry to disk
mv "$ARCHIVE_NAME.bz2" "/tmp/rootfs$ARCHIVE_NAME.bz2"

cd || return 1

}

error_harmless(){

ERROR=$1
SECOND_MESSAGE=$2

echo "##"
echo "##"
echo "## INSTALLATION FAILED!"
echo "##"
echo "## $ERROR"
echo "##"
echo "##"
echo "## THE INSTALLATION FAILED BUT NO"
echo "## DAMAGE WAS MADE, YOU CAN SAFELY"
echo "## SHUTDOWN YOUR DEVICE"
echo "##"
echo "## POST THE FULL LOG OF THIS INSTALLATION"
echo "## ON FORUMS FOR FURTHER ASSISTENCE"
echo "##"
echo "## $SECOND_MESSAGE"


}

error_dont_reboot(){

ERROR=$1
SECOND_MESSAGE=$2

echo "##"
echo "##"
echo "## INSTALLATION FAILED!"
echo "##"
echo "## $ERROR"
echo "##"
echo "##"
echo "## YOUR DEVICE MAY BE IN AN UNUSABLE STATE."
echo "## DO NOT REBOOT OR POWER OFF YOUR DEVICE"
echo "##"
echo "## POST THE FULL LOG OF THIS INSTALLATION"
echo "## ON FORUMS FOR FURTHER ASSISTENCE"
echo "##"
echo "## $SECOND_MESSAGE"


}

end_installation_message(){
echo ""
echo ""
echo ""
echo ""
echo "SUCESS! EVERYTHING DONE CORRECTLY! :D"
echo ""
echo "You can now reboot your device into Debian."
echo "If your device does not start Debian after rebooting,"
echo "you may need to restart the device by disconnecting the power."
echo ""
echo "The new root password is 'root'  Please change it immediately after"
echo "logging in."
echo ""


}

###############################################
#####
##### device or vendor-specific functions
#####
###############################################

zyxel_kill_programs(){

#this must be used for all zyxel devices as the firmware is pretty much the same for all

## killing the zyxel drive automounter daemon that %&£%"£ up everything any time it detects a new partition
#something else is still active and mounts stuff at boot or on hotplug but it does not annoy us otherwise
#killing also other stuff that might be annoying.
killall zyshd
killall myhotplug
killall zyfw_downloader
}

pogoplug_kill_programs(){

#this must be used for all pogoplug E02, pogoplug v4/mobile dockstar, sheevaplug devices as the firmware is pretty much the same for all

killall hbwd
}

###############################################
#####
##### functions that run the whole install process
##### they basically contain a list of macrofunctions above
##### this is to keep tidy the case -- esac below
#####
###############################################

install_rootfs(){
download_tools
install_formatting_tools
partition_and_format_disk
download_and_write_rootfs
}

install_rootfs_and_uboot(){
download_tools

check_bad_blocks

decide_what_drive_we_work_with

install_formatting_tools
#running the make rootfs only if $MAKE_ROOTFS != "NO"
if [ "$MAKE_ROOTFS" != "NO"  ]; then
partition_and_format_disk
download_and_write_rootfs
fi
remove_formatting_tools

install_uboot_tools
backing_up_uboot_and_envs
preserve_crucial_envs_for_later
install_envs
set_crucial_envs

#install uboot MUST be kept last, so that errors in partitioning or writing the rootfs or envs don't require recovery
install_uboot

end_installation_message
}

###############################################
#####
##### DISABLED FUNCTIONS, dead code that might be useful in future
#####
###############################################


disabled_uboot_check () {


# Dump existing uBoot and compare it to a database of known versions
###########
# we don't have stock uboots of most devices, so disabling this for now
##########

echo ""
echo "# Validating existing uBoot..."

# dump the first 512k of mtd0 to /tmp
$NANDDUMP --noecc --omitoob -l 0x80000 -f /tmp/uboot-mtd0-dump /dev/mtd0

wget -O "/tmp/valid-uboot.md5" "$VALID_UBOOT_MD5"

UPDATE_UBOOT=1
UBOOT_PLATFORM=
CURRENT_UBOOT_MD5=$(md5sum "/tmp/uboot-mtd0-dump" | cut -d' ' -f1)
UBOOT_DETAILS=$(grep "$CURRENT_UBOOT_MD5" /tmp/valid-uboot.md5 | head -1)
if [ "$UBOOT_DETAILS" != "" ]; then
  UBOOT_PLATFORM=$(echo "$UBOOT_DETAILS" | sed 's/^\w* \(\w*\) .*$/\1/')
  UBOOT_VERSION=$(echo "$UBOOT_DETAILS" | sed 's/^\w* \w* \(.*\)$/\1/')
  echo "## Valid uBoot detected: [$UBOOT_PLATFORM $UBOOT_VERSION]"
else
  echo "## Unknown uBoot detected on mtd0: $CURRENT_UBOOT_MD5"
  echo "##"
  if [ "$NO_UBOOT_CHECK" != "1" ]; then
    echo "## The installer could not detect the version of your current uBoot"
    echo "## This may happen if you have installed a different uBoot on"
    echo "## /dev/mtd0 or if you have bad blocks on /dev/mtd0"
    echo "##"
    echo "## If you have bad blocks on mtd0, you should not try to install uBoot."
#    echo "##"
#    echo "## If you have installed a diffirent uBoot on mtd0, and understand the"
#    echo "## risks, you can re-run the installer with the --no-uboot-check parameter"
    echo "##"
    echo "## Installation cancelled."
    rm "/tmp/valid-uboot.md5"
    exit 1
  else
    echo "## --no-uboot-check flag detected, continuing installation"

    while [ "$UBOOT_PLATFORM" = "" ]; do
      echo ""
      echo "############################################"
      echo "Your device could not be auto-detected."
      echo ""
      echo "You must be using a device listed below to run this installer."
      echo ""
      echo "What device are you using? Type the number of your device and press ENTER."
      echo "1 - Seagate Dockstar"
      echo "2 - Seagate GoFlex Net"
      echo "3 - Seagate GoFlex Home"
      echo "4 - Pogoplug v1"
      echo "5 - Pogoplug v2 - Pink"
      echo "6 - Pogoplug v4 or Mobile"
      echo "7 - Other"
      read -r device

      if [ "$device" = "1" ]; then
        echo "Selected Dockstar"
        UBOOT_PLATFORM="dockstar"
        UBOOT_VERSION="unknown"
      elif [ "$device" = "2" ]; then
        echo "Selected Seagate GoFlex Net"
        UBOOT_PLATFORM="goflexnet"
        UBOOT_VERSION="unknown"
      elif [ "$device" = "3" ]; then
        echo "Selected Seagate GoFlex Home"
        UBOOT_PLATFORM="goflexhome"
        UBOOT_VERSION="unknown"
      elif [ "$device" = "4" ]; then
        echo "Selected Pogoplug v1"
        UBOOT_PLATFORM="pogov1"
        UBOOT_VERSION="unknown"
      elif [ "$device" = "5" ]; then
        echo "Selected Pogoplug v2 - Pink"
        UBOOT_PLATFORM="pinkpogo"
        UBOOT_VERSION="unknown"
    elif [ "$device" = "6" ]; then
        echo "Selected Pogoplug v4 and Mobile"
        UBOOT_PLATFORM="pogo_v4"
        UBOOT_VERSION="unknown"
      elif [ "$device" = "7" ]; then
        echo "Selected Other Device, exiting"
        echo "This installer is only compatible with the listed devices."
        exit 1
      else
        echo "Invalid Input"
      fi
    done

  fi
fi

if [ "$FORCE_PLATFORM" != "" ] && [ "$FORCE_PLATFORM" != "$UBOOT_PLATFORM" ]; then
  echo "## --force-platform paramater [$FORCE_PLATFORM] does not match detected platform [$UBOOT_PLATFORM]."
  echo "Are you sure your device is a $FORCE_PLATFORM? [y/N] "
  read -r FORCE
  if [ "$FORCE" = "y" ] || [ "$FORCE" = "Y" ]; then
    echo "## Forcing installation of [$FORCE_PLATFORM] platform"
    UBOOT_PLATFORM=$FORCE_PLATFORM
    UBOOT_VERSION="unknown"
  else
    echo "## Exiting. No changes were made to mtd0."
    exit 1
  fi
fi

UBOOT_IS_CURRENT=$(echo $UBOOT_VERSION | grep -c current)
if [ "$UBOOT_IS_CURRENT" = "1" ]; then
  echo "## The newest uBoot is already installed on mtd0."
  UPDATE_UBOOT=0
else
  UBOOT_CURRENT=$(grep "$UBOOT_PLATFORM" /tmp/valid-uboot.md5 | grep current | head -1 | sed 's/^\w* \w* \(.*\)-current$/\1/')
fi

rm "/tmp/valid-uboot.md5"

}

disabled_service_disabler() {

# If this is the first time this installer has been run in the
# original Pogoplug enviroment, check if the user wants to disable
# the Pogoplug services
#########
#this stuff will likely be split into device-specific functions,
#for now it stays disabled
#########

if [ -d /usr/local/cloudengines/ ] && [ ! -e "$UBOOT_ORIGINAL" ]; then
  killall hbwd
  echo ""
  echo ""
  echo ""
  echo "DISABLE POGOPLUG SERVICES"
  echo ""
  echo "The pogoplug service includes an auto-update feature which could"
  echo "be used to cripple or disable your device.  It is recommended"
  echo "that you disable this service."
  echo ""
  echo "NOTE: The pogoplug service is proprietary software"
  echo "created by Cloud Engines.  It is not available for use"
  echo "in other distributions and will not be available in"
  echo "your new linux installation even if you choose not to disable it."
  echo ""
  echo "Would you like to disable the pogoplug services? [Y/n] "
  read -r DISABLE
  if [ "$DISABLE" = "" ] || [ "$DISABLE" = "y" ] || [ "$DISABLE" = "Y" ];
  then

    echo "Applying fixes to the pogoplug environment..."

    mount -o rw,remount /

    # Add /sbin to the path and cleanup the shell prompt
    if [ ! -f "/root/.bash_profile" ]; then
      echo -e \
  "export PS1='\h:\w\$ '
  export PATH='/usr/bin:/bin:/sbin'
  " > /root/.bash_profile
    fi

    chmod go+w /dev/null

    # Re-enable dropbear (updated dockstars only)
    sed -i 's/^#\/usr\/sbin\/dropbear/\/usr\/sbin\/dropbear/' /etc/init.d/db > /dev/null 2>&1

    echo "Disabling the pogoplug service..."
    # Comment out the line that starts hmbgr.sh
    sed -i 's/^\/etc\/init.d\/hbmgr.sh start/#Uncomment the line below to enable the pogoplug service\n#\/etc\/init.d\/hbmgr.sh start/' /etc/init.d/rcS

    mount -o ro,remount /

    echo "Done fixing pogoplug environment."
    echo ""
  fi

  UBOOT_ORIGINAL_URL="$UBOOT_MTD0_BASE_URL.$UBOOT_PLATFORM.original.kwb"
  install "$UBOOT_ORIGINAL"   "$UBOOT_ORIGINAL_URL"    644

  #install "$BLPARAM"          "$BLPARAM_URL"           755 we don't have blparam

  if   [ "$UBOOT_PLATFORM" = "pinkpogo"  ]; then BOOTCMD='nand read.e 0x800000 0x100000 0x200000; setenv bootargs $(console) $(bootargs_root); bootm 0x800000'
  # dockstar, goflex, and pogov1 have the same bootcmd
  else BOOTCMD='nand read.e 0x800000 0x100000 0x300000; setenv bootargs $(console) $(bootargs_root); bootm 0x800000'
  fi
  #$BLPARAM "bootcmd=$BOOTCMD" > /dev/null 2>&1

  # Preserve the MAC address
  #ENV_ETHADDR=`$BLPARAM | grep "^ethaddr=" | cut -d'=' -f 2-`
fi
}

disabled_preserve_custom_envs_for_later() {

#this is not working, currently not used

  # Preserve the 'rescue_installed' setting
  ENV_RESCUE_INSTALLED=`$FW_PRINTENV rescue_installed 2> /dev/null | cut -d'=' -f 2-`
  if [ "$ENV_RESCUE_INSTALLED" = "" ]; then
    ENV_BOOTCMD_RESCUE=`$FW_PRINTENV bootcmd_rescue 2> /dev/null`
    if [ "$ENV_BOOTCMD_RESCUE" != "" ]; then
      ENV_RESCUE_INSTALLED=1
    fi
  fi

  # Preserve the arcNumber value
  ENV_ARCNUMBER=`$FW_PRINTENV arcNumber 2> /dev/null | cut -d'=' -f 2-`

  # Preserve the custom kernel parameters
  ENV_RESCUE_CUSTOM=`$FW_PRINTENV rescue_custom_params 2> /dev/null | cut -d'=' -f 2-`
  ENV_USB_CUSTOM=`$FW_PRINTENV usb_custom_params 2> /dev/null | cut -d'=' -f 2-`
  ENV_UBIFS_CUSTOM=`$FW_PRINTENV ubifs_custom_params 2> /dev/null | cut -d'=' -f 2-`
}

disabled_reload_custom_envs(){

# not working and not used, writing custom envs again from backup

  $FW_SETENV ethaddr $ENV_ETHADDR
  if [ "$ENV_RESCUE_INSTALLED" = "1" ]; then $FW_SETENV rescue_installed $ENV_RESCUE_INSTALLED; fi
  if [ "$ENV_RESCUE_CUSTOM" != "" ]; then $FW_SETENV rescue_custom_params $ENV_RESCUE_CUSTOM; fi
  if [ "$ENV_USB_CUSTOM" != "" ]; then $FW_SETENV usb_custom_params $ENV_USB_CUSTOM; fi
  if [ "$ENV_UBIFS_CUSTOM" != "" ]; then $FW_SETENV ubifs_custom_params $ENV_UBIFS_CUSTOM; fi
}

#this works fine, but until we have a sgdisk we can't use it
GPT_MBR_partition_and_format_disk() {

#by default the device to use is this
DISK="/dev/sda"

#checking if there are drives over a GiB at all
DETECTED_DRIVE=$(sgdisk -p /dev/sda | grep /dev | grep GiB )

#checking if we see a SDcard in a true SDcard slot over a GiB in size
DETECTED_MMC=$(sgdisk -p /dev/mmcblk0 | grep /dev | grep GiB )

#if both searches turn out empty, we have no drives to work with
if [[ "$DETECTED_DRIVE" = "" ]] && [[ "$DETECTED_MMC" = "" ]] ; then
  echo "No disks or SD cards detected. Aborting!"
  exit 1
fi

#if no drives are detected but a SDcard is detected, we select that for partitioning
if [[ "$DETECTED_DRIVE" = "" ]] && [[ "$DETECTED_MMC" != "" ]] ; then
  DISK="/dev/mmcblk0"
fi

#checking if there is another drive or if we detected one SDcard and no disks above so we can skip questions
DETECTED_DRIVE=$(sgdisk -p /dev/sdb | grep /dev | grep GiB  )
if [[ "$DETECTED_DRIVE" = "" ]] || [[ "$DISK" = "/dev/mmcblk0" ]] ; then
  #if conditions are true we skip the else block and proceed with /dev/sda
  DISK="/dev/sda"
else
  #ok, there is more than one drive in this box.
  #since for various reasons we cannot get a RELIABLE way to find what device is friggin /
  #and in case of multiple drives it makes more sense anyway, we ask the user what drive he wants

  echo
  echo
  echo "what drive you want to install Debian in? "
  echo "(write 1 or 2 or 3 or 4 or sd to select the drive)"
  echo "(write Q if you want to quit)"
  echo
  DISK=$(sgdisk -p /dev/sda | grep /dev | grep GiB  )
  if [[ "$DISK" == *"Gib"* ]]; then
    echo "Disk 1 size: $(echo "$DISK" | awk '{ print $5, $6 }')"
  fi

  DISK=$(sgdisk -p /dev/sdb | grep /dev | grep GiB  )
  if [[ "$DISK" == *"Gib"* ]]; then
    echo "Disk 2 size: $(echo "$DISK" | awk '{ print $5, $6 }')"
  fi

  DISK=$(sgdisk -p /dev/sdc | grep /dev | grep GiB  )
  if [[ "$DISK" == *"Gib"* ]]; then
    echo "Disk 3 size: $(echo "$DISK" | awk '{ print $5, $6 }')"
  fi

  DISK=$(sgdisk -p /dev/sdd | grep /dev | grep GiB  )
  if [[ "$DISK" == *"Gib"* ]]; then
    echo "Disk 4 size: $(echo "$DISK" | awk '{ print $5, $6 }')"
  fi

  DISK=$(sgdisk -p /dev/mmcblk0 | grep /dev | grep GiB  )
  if [[ "$DISK" == *"Gib"* ]]; then
    echo "SDcard size: $(echo "$DISK" | awk '{ print $5, $6 }')"
  fi

  read -r SELECTION

  case $SELECTION in

    1)
      DISK="/dev/sda"
    ;;
    2)
      DISK="/dev/sdb"
    ;;
    3)
      DISK="/dev/sdc"
    ;;
    4)
      DISK="/dev/sdd"
    ;;
    sd)
      DISK="/dev/mmcblk0"
    ;;
  esac
fi

echo
echo "installation drive is $DISK"
echo

PART_1="${DISK}1"
PART_2="${DISK}2"

echo
echo "---unmounting installation drive"
echo

#deactivating raids
for f in "/dev/md0" "/dev/md1" "/dev/md2" "/dev/md3" "/dev/md4" ; do

  umount -l "$f"
  export PATH="$ORIGINAL_PATH"
  mdadm --stop "$f"
  export PATH="$TOOLS"
done

#unmounting plain partitions if they were automounted
umount -l "$PART_1"
umount -l "$PART_2"

echo
echo "---partitioning installation drive"
echo

#checking install drive size
DRIVE_SIZE=$(sgdisk -p $DISK | grep /dev | awk '{print $5}')

echo "install_drive_size is $DRIVE_SIZE"

# turning the size value into an integer
DRIVE_SIZE_INTEGER=$( echo "$DRIVE_SIZE" | sed 's/\..*//' )

echo "DRIVE_SIZE_INTEGER is $DRIVE_SIZE_INTEGER"

#doing a bunch of checks to decide what partition layout to create.

if (( "$DRIVE_SIZE_INTEGER" < 33 )); then

  echo "drive is smaller than 33GB so we assume this is a flash drive."
  echo "we will create only a system partition, no data partition."

  #checking if the user called the script with GPT option
  if [[  $PARTITION_TABLE = "GPT" ]]; then

    sgdisk -o -n1:64:0 -t1:8300 -g $DISK

  else
    #else we partition in MBR
    fdisk "$DISK" <<EOF
o
n
p
1


w
EOF
  fi
fi

if (( "$DRIVE_SIZE_INTEGER" < 2048 )); then

  echo "drive is bigger than 33GB but smaller than 2TB so we assume this is a hard drive."
  echo "making a 7 GB system partition, the rest becomes a data partition"

  NEED_DATA="yes"

  #checking if the user called the script with GPT option
  if [[  $PARTITION_TABLE = "GPT" ]]; then

      sgdisk -o -n1:64:+7G -n2:0:0 -t1:8300 -t2:8300 -g $DISK

    else
      #else we partition in MBR
      fdisk "$DISK" <<EOF
o
n
p
1

+7168M
n
p
2


w
EOF
  fi

fi

if (( "$DRIVE_SIZE_INTEGER" > 2048 )); then

  echo "drive is bigger than 2TB so we must format it as GPT"
  echo "only the new uboot can boot drives formatted as GPT, original uboot cannot"
  echo "making a 7 GB system partition, the rest becomes a data partition"

  NEED_DATA="yes"

  sgdisk -o -n1:64:+7G -n2:0:0 -t1:8300 -t2:8300 -g $DISK

fi

echo "---pausing for a bit so ioctl can re-sync partitions"
sleep 10

#  make filesystems
echo "---formatting the partitions we created"

if [ -f "/sbin/e2label" ]; then
  # this means we are inside a Debian rootfs,
  # labelling it now as rootfs will screw up mounting later
  # it will be labelled later when unmounted
  mke2fs -t "$FILESYSTEM" "$PART_1"

else
  #we are not in a Debian rootfs, so we can give it a label only this way
  mke2fs -t "$FILESYSTEM" -L "rootfs" "$PART_1"
fi

#checking if swap and data partition is needed
if [ "$NEED_DATA" = "yes" ]; then
  mke2fs -t "ext4" -L "data" "$PART_2"
fi

echo '---doing file system check on partitons to be sure that they are ok'

e2fsck -fp "$PART_1"

if [ "$NEED_DATA" = "yes" ]; then
  e2fsck -fp "$PART_2"
fi
}

################################################
#####
##### MAIN
#####
################################################

#doing a root check, because the installer must be run as root for obvious reasons
if [ "$(id -u)" -eq 0 ]; then
	echo ;
else
	echo "you must run this script as root";
exit
fi


###default settings for script operation
CUSTOM_PARAM=""
MAKE_ROOTFS="YES"

#filesystem to use when formatting
FILESYSTEM="ext3"

##loading advanced options passed as arguments if any, and changing relevant variables
for i in "$@"
do
case $i in
    -GPT) #this is there but it is not used
      PARTITION_TABLE="GPT"
    ;;
    -ext4)
      FILESYSTEM="ext4"
    ;;
    -systemd)
    CUSTOM_PARAM="$CUSTOM_PARAM init=/bin/systemd"
    ;;
    -no-rootfs)
      MAKE_ROOTFS="NO"
    ;;
    *)
            # unknown options are ignored
    ;;
esac
done


echo ""
echo ""
echo "!!!!!!  WARNING: PLEASE READ THIS CAREFULLY  !!!!!!"
echo ""
echo "To boot Debian we will replace the bootloader in the "
echo "internal memory of the device"
echo ""
echo "Due to bootloader limitations, booting the stock firmware"
echo "will usually not be possible afterwards."
echo "Booting third party firmwares like OpenWRT or LEDE will work"
echo "but installing them is outside the scope of this installer."
echo "At least for now."
echo ""
echo "If you lose power while the bootloader is being flashed,"
echo "your device could be left in an unusable state and require"
echo "serial connection to recover, please ask for help in the forums."
echo ""
echo "Flashing the bootloader is very fast though, and this script spends"
echo "most of the time doing other jobs like installing Debian."
echo ""
echo "We need a storage device to install Debian in as the onboard"
echo "storage of most devices is tiny, please make sure that there is a"
echo "USB, Sata or SDcard storage device we can install Debian in."
echo "We need at least 2GB of storage space, the faster storage the better."
echo ""
echo "This script will DESTROY ALL EXISTING DATA ON THE SELECTED STORAGE DEVICE"
echo "so move all your data off any drives please."
echo ""
echo ""
echo "By typing ok, you agree to assume all liabilities and risks"
echo "associated with running this installer."
echo ""
echo "If everything looks good, type 'ok' to continue: "


read -r IS_OK
if [ "$IS_OK" != "OK" ] && [ "$IS_OK" != "Ok" ] && [ "$IS_OK" != "ok" ]
then
  echo "Exiting..."
  exit
fi

#SELECTING DEVICE
echo "Good, now we need to know what device we are working on"
echo "here is a list, please write the number of the device"

#add here all devices supported
echo "0.For Expert Users"
echo "1.Zyxel nsa310"
echo "2.Zyxel nsa325 (v1 and v2)"
echo "3.Pogoplug V4 (Series 4 and Mobile)"

#loading the user selection
read -r DEVICE_TYPE

# and here goes all device-specific operations
case "$DEVICE_TYPE" in

    0)  # For Expert Users
      echo "00. download and unpack toolbox"
      echo "01. make a Debian rootfs in a drive"
      echo "02. gathering info about this device"
      #loading the user selection
      read -r DEVICE_TYPE

      # and here goes all device-specific operations
      case "$DEVICE_TYPE" in

        00)    echo "downloading tools"
          download_tools
          install_formatting_tools
          install_uboot_tools
          echo ""
          echo ""
          echo "tools downloaded in $TOOLS, busybox tools installed"
          echo "you may want to write " "export PATH=$TOOLS:$PATH"
          echo "so you can use them as default tools for this session"
        ;;

        01)    echo "downloading tools and making a rootfs"
          install_rootfs
        ;;

        02) echo "gathering info about this device"
          download_tools
          install_uboot_tools
          gather_telemetry
        ;;

      esac
    ;;

	1) #zyxel nsa310
		PLATFORM="nsa310" # platform name used inside uboot name
		STOCK_MTD0="/dev/mtd0" #this is the how the mtd0 is, as seen from inside the stock firmware
		ENV_DTB_FILE="/boot/dts/kirkwood-nsa310b-lmsensor-redusbled.dtb" # this is the env to add the dtb
		MACHID="" # leave empty if none
		ARCNUMBER="4022" # leave empty if none

		# the uboot download link variable is written above as uboot links are likely to change
		# more often than most of the data here, that should be write-once-and-never-touch-again
		UBOOT_URL=$UBOOT_NSA310_URL

        #running a killer function to kill off a bunch of stock FW daemons that get in the way
        zyxel_kill_programs

        install_rootfs_and_uboot

	;;
	2) #zyxel nsa325
		PLATFORM="nsa325"
		UBOOT_URL=$UBOOT_NSA325_URL
		STOCK_MTD0="/dev/mtd0" #this is the how the mtd0 is, as seen from inside the stock firmware
		ENV_DTB_FILE="/boot/dts/kirkwood-nsa325.dtb"
		MACHID="" # leave empty if none
		ARCNUMBER="4495" # leave empty if none

		# the uboot download link variable is written above as uboot links are likely to change
		# more often than most of the data here, that should be write-once-and-never-touch-again
		UBOOT_URL=$UBOOT_NSA325_URL

        #running a killer function to disable features that get in the way
        zyxel_kill_programs

		install_rootfs_and_uboot
	;;
	3) #Pogoplug V4/Mobile
		PLATFORM="pogo_v4"
        UBOOT_URL=$UBOOT_POGO_V4_URL
		STOCK_MTD0="/dev/mtd0" #this is the how the mtd0 is, as seen from inside the stock firmware
		ENV_DTB_FILE="'/boot/dts/kirkwood-pogoplug_v4.dtb'"
        MACHID="F78" # leave empty if none
        ARCNUMBER="3960" # leave empty if none

        # the uboot download link variable is written above as uboot links are likely to change
        # more often than most of the data here, that should be write-once-and-never-touch-again
        UBOOT_URL=$UBOOT_POGO_V4_URL

		#device-specific commands to be run before proceding
        pogoplug_kill_programs

		install_rootfs_and_uboot

	;;

esac

#getting the path back to default
export PATH="$ORIGINAL_PATH"

echo "Reboot now? [Y/n] "
read -r IN
if [ "$IN" = "" ] || [ "$IN" = "y" ] || [ "$IN" = "Y" ]
then
  /sbin/reboot
fi

exit 0


